"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const NO_INFO = null;
function highlightWith(c) {
    return compose(highlightVerifiedWith(c.verify, c.markWith(c.identifiers.verified)(NO_INFO)), highlightMistakesWith(c.mistakes, c.markWith(c.identifiers.mistake)));
}
exports.highlightWith = highlightWith;
function highlightMistakesWith(mistakes, mark) {
    function replacer(info, substring) {
        const transform = (substring === undefined
            ? mark(info)
            : (s) => s.split(substring).join(mark(info)(substring)));
        return (match, ...rest) => {
            const captured = rest.find(x => x !== undefined);
            return (typeof captured === "string"
                ? match.replace(captured, transform(captured))
                : transform(match));
        };
    }
    const transformers = mistakes.map(m => (text) => text.replace(join(m.contexts), replacer(m.info || null, m.substring)));
    return transformers.reduceRight(compose, x => x);
}
exports.highlightMistakesWith = highlightMistakesWith;
function highlightVerifiedWith(pattern, mark) {
    return text => text.replace(pattern, mark);
}
exports.highlightVerifiedWith = highlightVerifiedWith;
function compose(f, g) {
    return x => f(g(x));
}
function join(patterns) {
    return new RegExp(patterns.map(p => p.source).join("|"), "g");
}
//# sourceMappingURL=index.js.map